@@ -673,6 +673,7 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
             break;
         case _equirectangular:
         case _fisheye_ff:
+        case _fisheye_circ:
         case _panorama:
         case _mercator:
         case _sinusoidal:
@@ -702,18 +703,20 @@ void SetMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im , Imag
         {
         case _rectilinear:
             // calculate distance for this projection
-            mp->scale[0] = (double) im->width / (2.0 * tan(a/2.0)) / mp->distance;
+            mp->scale[0] = (double) image_selection_width / (2.0 * tan(a/2.0)) / mp->distance;
             break;
         case _equirectangular:
         case _panorama:
         case _fisheye_ff:
         case _fisheye_circ:
-            mp->scale[0] = ((double) im->width) / a / mp->distance;
+        case _mercator:
+        case _sinusoidal:
+            mp->scale[0] = ((double) image_selection_width) / a / mp->distance;
             break;
         default:
             PrintError ("SetMakeParams: Unsupported input image projection");
             // no way to report an error back to the caller...
-            mp->scale[1] = 1;
+            mp->scale[0] = 1;
         }
         mp->scale[1]    = mp->scale[0];
 
@@ -903,6 +906,7 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
             break;
         case _equirectangular:
         case _fisheye_ff:
+        case _fisheye_circ:
         case _panorama:
         case _mercator:
         case _sinusoidal:
@@ -921,7 +925,7 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
             break;
         default:
             // unknown
-            PrintError ("SetMakeParams: Unsupported panorama projection");
+            PrintError ("SetInvMakeParams: Unsupported panorama projection");
             // no way to report an error back to the caller...
             mp->distance = 1;
         }
@@ -938,12 +942,14 @@ void 	SetInvMakeParams( struct fDesc *stack, struct MakeParams *mp, Image *im ,
         case _panorama:
         case _fisheye_ff:
         case _fisheye_circ:
+        case _mercator:
+        case _sinusoidal:
             mp->scale[0] = ((double) im->width) / a / mp->distance;
             break;
         default:
-            PrintError ("SetMakeParams: Unsupported input image projection");
+            PrintError ("SetInvMakeParams: Unsupported input image projection");
             // no way to report an error back to the caller...
-            mp->scale[1] = 1;
+            mp->scale[0] = 1;
         }
         mp->scale[1]    = mp->scale[0];
 
@@ -1122,7 +1128,7 @@ void 	SetInvMakeParamsCorrect( struct fDesc *stack, struct MakeParams *mp, Image
         if(im->cP.cutFrame)
         {
 			imSel.width = im->selection.right  - im->selection.left;
-			imSel.width = im->selection.bottom - im->selection.top;
+			imSel.height = im->selection.bottom - im->selection.top;
 
 			mp->horizontal += (im->selection.right  + im->selection.left - im->width)/2.0;
 			mp->vertical   += (im->selection.bottom + im->selection.top  - im->height)/2.0;
