@@ -549,6 +549,7 @@ void PT_setInfoDlgFcn(int (*ptr)(int, char *));            // set custom info ca
 int 	infoDlg ( int command, char* argument );	// Display info: same argumenmts as progress
 void PT_setErrorFcn( void (*ptr)( char* , va_list va));         // set custom error function
 void  	PrintError( char* fmt, ...);				// Error Reporting
+void dieWithError(char*fmt, ...);
 
 int 	ccommand( char ***argvPtr);					// Shell for standalone programs
 
@@ -615,7 +616,8 @@ void 	writePrefs			(char* p, int selector );   			// Preferences, same selector
 
 int	LoadBufImage		( Image *image, char *fname, int mode);
 int	SaveBufImage		( Image *image, char *fname );
-int	writeTIFF			( Image *im, fullPath* fname);			// On Mac: fname is FSSpec*				
+int writeCroppedTIFF    ( Image *im, fullPath *sfile, CropInfo *crop_info);
+int	writeTIFF			( Image *im, fullPath *fname);			// On Mac: fname is FSSpec*				
 void 	SaveOptions			( struct correct_Prefs * thePrefs );
 int 	LoadOptions			( struct correct_Prefs * thePrefs );
 void  	FindScript			( struct adjust_Prefs *thePrefs );
@@ -925,9 +927,15 @@ extern sPrefs			*gsPrPtr;
 	#define		read_bin			"rb"
 	#define		read_write_text			"rw"
 	#define		append_bin			"ab"
-	#define		p2cstr( x )	
-	#define		c2pstr( x )
-															
+    
+    // Ippei hack. OSX with GCC+ANSI mode.
+    #ifdef MAC_OS_X_VERSION_10_4
+        // MacOSX 10.4 has those functions predefined in Carbon API.
+        #include <Carbon/Carbon.h> // CoreServices/TextUtils.h
+    #else
+        #define		p2cstr( x )	
+        #define		c2pstr( x )
+    #endif
 
 #endif
 
