@@ -23,11 +23,12 @@ typedef struct {char* name; char* value;}  TStringFeature;
 TIntFeature intFeatures[] ={
   {"CPErrorIsDistSphere",1},       // optimizer reports angular control point errors
   {"NumLensTypes",5},              // source lens types 0..4
-  {"NumPanoTypes",4},              // pano lens types 0..3
+  {"NumPanoTypes",8},              // pano lens types 0..7
   {"CanCropOutside",1},
   {"CanHaveNegativeCP",1},
   {"AntiAliasingFilter",1},
-  {"NumFilter",24}
+  {"NumFilter",24},
+  {"SetProgressFcn",1}             // setProgressFcn and setInfoDlgFcn are available
 };
 
 TDoubleFeature doubleFeatures[] ={
@@ -59,6 +60,10 @@ TStringFeature stringFeatures[]={
   {"PanoType1","Cylindrical"},
   {"PanoType2","Equirectangular"},
   {"PanoType3","Full Frame"},
+  {"PanoType4","Stereographic"},
+  {"PanoType5","Mercator"},
+  {"PanoType6","Transverse mercator"},
+  {"PanoType7","Sinusoidal"},
   // Filter Types
   //   fix: Fixed Windowsize
   //   aa: Antialiasing filter with adaptive filter size
@@ -116,6 +121,7 @@ TStringFeature stringFeatures[]={
   {"BMPrev", "Jim Watters, correctly open BMP files created with rows in reverse order"},
   {"Tiff32", "Thomas Rauscher, Load and save TIFF 32-bit with IEEE floats, http://www.pano2qtvr.com/dll_patch/"},
   {"AntiAliasing", "Thomas Rauscher, New antialiasing filter for adjust, http://www.pano2qtvr.com/dll_patch/"},
+  {"HDRFile", "Thomas Rauscher, Load and save Radiance HDR files, http://www.pano2qtvr.com/dll_patch/"},
   {"Patch200505", "Douglas Wilkins, Correct behaviour when mode = _usedata"},
 #ifdef HasJava
   {"Patch200504a", "Douglas Wilkins, Java support enabled"},
