@@ -21,11 +21,11 @@
 #include "panotypes.h"
 #include <signal.h>
 
+#define PREF_FILE_NAME "pano13.prf"
 
 //------------------ Public functions required by filter.h -------------------------------
 
 
-
 void filter_main( TrformStr *TrPtr, struct size_Prefs *spref)
 {
 	dispatch	( TrPtr, spref);
@@ -129,7 +129,7 @@ int readPrefs( char* pref, int selector )
 {
 
 	struct {
-		char						v[sizeof(VERSION)];
+		char						v[sizeof(PREF_VERSION)];
 		struct correct_Prefs		c;
 		struct remap_Prefs			r;
 		struct perspective_Prefs	p;
@@ -137,7 +137,7 @@ int readPrefs( char* pref, int selector )
 		struct size_Prefs			s;
 		panControls					pc;
 	} prf;
-	char* prefname = "pano12.prf";
+	char* prefname = PREF_FILE_NAME;
 	long size;
 
 	FILE 	*prfile;
@@ -153,7 +153,7 @@ int readPrefs( char* pref, int selector )
 		}else{
 			switch( selector){
 				case _version:
-					memcpy( pref, &prf.v, sizeof( VERSION ) );
+					memcpy( pref, &prf.v, sizeof( PREF_VERSION ) );
 					break;
 				case _correct:
 					if( prf.c.magic != 20 ) 
@@ -212,7 +212,7 @@ int readPrefs( char* pref, int selector )
 void writePrefs( char* prefs, int selector ){
 
 	struct {
-		char						v[sizeof(VERSION)];
+		char						v[sizeof(PREF_VERSION)];
 		struct correct_Prefs		c;
 		struct remap_Prefs			r;
 		struct perspective_Prefs	p;
@@ -222,7 +222,7 @@ void writePrefs( char* prefs, int selector ){
 	} prf;
 
 	FILE 	*prfile;
-	char* prefname = "pano12.prf";
+	char* prefname = PREF_FILE_NAME;
 
 
 
@@ -233,7 +233,7 @@ void writePrefs( char* prefs, int selector ){
 
 	switch( selector){
 		case _version:
-			memcpy( &prf.v,  prefs, sizeof( VERSION ) );
+			memcpy( &prf.v,  prefs, sizeof( PREF_VERSION ) );
 			break;
 		case _correct:
 			memcpy( &prf.c , prefs, sizeof(struct correct_Prefs)); 
