@@ -71,6 +71,8 @@ caused annoying warnings about unknown tags while loading each tiff source image
 #define ID_MM  100
 #define ID_SM  101
 
+#define PREF_FILE_NAME "pano13.prf"
+
 HINSTANCE 	hDllInstance 	= NULL;
 HWND 		wndOwner 		= NULL;
 int 		dialogDone;
@@ -277,7 +279,7 @@ int readPrefs( char* pref, int selector )
 {
 
 	struct {
-		char						v[sizeof(VERSION)];
+		char						v[sizeof(PREF_VERSION)];
 		struct correct_Prefs		c;
 		struct remap_Prefs			r;
 		struct perspective_Prefs	p;
@@ -296,11 +298,11 @@ int readPrefs( char* pref, int selector )
 	if( c!= NULL )
 	{
 		c++;
-		strcpy( c,"pano12.prf" );
+		strcpy( c,PREF_FILE_NAME );
 	}
 	else
 	{
-		strcpy( prefname,".\\pano12.prf" );
+		strcpy( prefname,".\\"PREF_FILE_NAME );
 	}
 
 
@@ -319,7 +321,7 @@ int readPrefs( char* pref, int selector )
 			switch( selector)
 			{
 				case _version:
-					memcpy( pref, &prf.v, sizeof( VERSION ) );
+					memcpy( pref, &prf.v, sizeof( PREF_VERSION ) );
 					break;
 				case _correct:
 					if( prf.c.magic != 20 ) 
@@ -379,7 +381,7 @@ void writePrefs( char* prefs, int selector )
 {
 
 	struct {
-		char						v[sizeof(VERSION)];
+		char						v[sizeof(PREF_VERSION)];
 		struct correct_Prefs		c;
 		struct remap_Prefs			r;
 		struct perspective_Prefs	p;
@@ -396,11 +398,11 @@ void writePrefs( char* prefs, int selector )
 	if( c!= NULL )
 	{
 		c++;
-		strcpy( c,"pano12.prf" );
+		strcpy( c,PREF_FILE_NAME );
 	}
 	else
 	{
-		strcpy( prefname,".\\pano12.prf" );
+		strcpy( prefname,".\\"PREF_FILE_NAME );
 	}
 
 
@@ -414,7 +416,7 @@ void writePrefs( char* prefs, int selector )
 	switch( selector)
 	{
 		case _version:
-			memcpy( &prf.v,  prefs, sizeof( VERSION ) );
+			memcpy( &prf.v,  prefs, sizeof( PREF_VERSION ) );
 			break;
 		case _correct:
 			memcpy( &prf.c , prefs, sizeof(struct correct_Prefs)); 
